@@ -12,8 +12,9 @@ from tinychain.util import form_of, uri, URI, to_json
 
 
 class Cluster(object, metaclass=Meta):
-    """A hosted Tinychain service."""
+    """A hosted TinyChain service."""
 
+    # TODO: get rid of this
     @classmethod
     def __use__(cls):
         instance = cls()
@@ -24,11 +25,11 @@ class Cluster(object, metaclass=Meta):
             if isinstance(attr, MethodStub):
                 setattr(instance, name, attr.method(instance, name))
             elif inspect.isclass(attr) and issubclass(attr, State):
-                if not uri(cls).startswith(uri(attr)):
+                if not uri(attr).startswith(uri(cls)):
                     logging.warning(f"cluster at {uri(cls)} serves class with different URI: {uri(attr)}")
 
                 @get_method
-                def ctr(self, txn, form) -> attr:
+                def ctr(self, form) -> attr:
                     return attr(form)
 
                 setattr(instance, name, ctr.method(instance, name))
