@@ -1,4 +1,4 @@
-"""Utilities for communicating with a Tinychain host."""
+"""Utilities for communicating with a TinyChain host."""
 
 import json
 import logging
@@ -11,6 +11,7 @@ import urllib.parse
 
 from tinychain.error import *
 from tinychain.util import to_json, uri, URI
+from tinychain.value import Nil
 
 
 DEFAULT_PORT = 8702
@@ -18,7 +19,7 @@ ENCODING = "utf-8"
 
 
 class Host(object):
-    """A Tinychain host."""
+    """A TinyChain host."""
 
     @staticmethod
     def encode_params(params):
@@ -36,7 +37,11 @@ class Host(object):
     def _handle(self, req):
         response = req()
         status = response.status_code
-        response = json.loads(response.text)
+
+        try:
+            response = json.loads(response.text)
+        except json.decoder.JSONDecodeError as cause:
+            raise ValueError(f"invalid JSON response: {response.text} ({cause}")
 
         if status == 200:
             return response
@@ -52,6 +57,10 @@ class Host(object):
             raise NotFound(response)
         elif status == 405:
             raise MethodNotAllowed(response)
+        elif status == 408:
+            raise Timeout(response)
+        elif status == 409:
+            raise Conflict(response)
         elif status == 501:
             raise NotImplemented(response)
         else:
@@ -67,7 +76,7 @@ class Host(object):
 
         url = self.link(path)
         headers = auth_header(auth)
-        if key:
+        if key and not isinstance(key, Nil):
             key = json.dumps(to_json(key)).encode(ENCODING)
             request = lambda: requests.get(url, params={"key": key}, headers=headers)
         else:
@@ -80,9 +89,14 @@ class Host(object):
 
         url = self.link(path)
         headers = auth_header(auth)
-        key = json.dumps(to_json(key)).encode(ENCODING)
+        if key and not isinstance(key, Nil):
+            key = json.dumps(to_json(key)).encode(ENCODING)
+            params = {"key": key}
+        else:
+            params = {}
+
         value = json.dumps(to_json(value)).encode(ENCODING)
-        request = lambda: requests.put(url, params={"key": key}, data=value, headers=headers)
+        request = lambda: requests.put(url, params=params, data=value, headers=headers)
 
         return self._handle(request)
 
@@ -101,7 +115,7 @@ class Host(object):
 
         url = self.link(path)
         headers = auth_header(auth)
-        if key:
+        if key and not isinstance(key, Nil):
             key = json.dumps(to_json(key)).encode(ENCODING)
             request = lambda: requests.delete(url, params={"key": key}, headers=headers)
         else:
@@ -111,7 +125,7 @@ class Host(object):
 
 
 class Local(Host):
-    """A local Tinychain host."""
+    """A local TinyChain host."""
 
     ADDRESS = "127.0.0.1"
     SHUTDOWN_TIME = 0.1
@@ -125,7 +139,8 @@ class Local(Host):
             port=DEFAULT_PORT,
             log_level="warn",
             cache_size="1G",
-            force_create=False):
+            force_create=False,
+            request_ttl="30"):
 
         # set _process first so it's available to __del__ in case of an exception
         self._process = None
@@ -150,6 +165,7 @@ class Local(Host):
             f"--http_port={port}",
             f"--log_level={log_level}",
             f"--cache_size={cache_size}",
+            f"--request_ttl={request_ttl}",
         ]
 
         if data_dir:
@@ -169,14 +185,14 @@ class Local(Host):
         time.sleep(wait_time)
 
         if self._process is None or self._process.poll() is not None:
-            raise RuntimeError(f"Tinychain process at {uri(self)} crashed on startup")
+            raise RuntimeError(f"TinyChain process at {uri(self)} crashed on startup")
         else:
             logging.info(f"new instance running at {uri(self)}")
 
     def stop(self, wait_time=SHUTDOWN_TIME):
         """Shut down this host."""
 
-        logging.info(f"Shutting down Tinychain host {uri(self)}")
+        logging.info(f"Shutting down TinyChain host {uri(self)}")
         if self._process:
             self._process.terminate()
             self._process.wait()
