@@ -9,7 +9,9 @@ def gen_headers(instance):
         if name.startswith('_'):
             continue
 
-        if isinstance(attr, MethodStub):
+        if isinstance(attr, Attribute):
+            setattr(instance, name, attr.method(instance, name))
+        elif isinstance(attr, MethodStub):
             setattr(instance, name, attr.method(instance, name))
 
 
@@ -19,7 +21,7 @@ class Meta(type):
     def __form__(cls):
         mro = cls.mro()
         if len(mro) < 2:
-            raise ValueError("Tinychain class must extend a subclass of State")
+            raise ValueError("TinyChain class must extend a subclass of State")
 
         parent_members = dict(inspect.getmembers(mro[1](URI("self"))))
 
@@ -31,10 +33,18 @@ class Meta(type):
         instance = cls(instance_uri)
 
         for name, attr in inspect.getmembers(instance):
-            if name.startswith('_'):
+            if name.startswith('_') or isinstance(attr, URI):
+                continue
+            elif hasattr(attr, "accessor"):
+                if attr.accessor:
+                    continue
+            elif inspect.ismethod(attr) and attr.__self__ is cls:
+                # it's a @classmethod
                 continue
 
-            if isinstance(attr, MethodStub):
+            if isinstance(attr, Attribute):
+                setattr(header, name, attr.method(instance, name))
+            elif isinstance(attr, MethodStub):
                 setattr(header, name, attr.method(instance, name))
             elif isinstance(attr, State):
                 setattr(header, name, type(attr)(instance_uri.append(name)))
@@ -43,14 +53,20 @@ class Meta(type):
 
         form = {}
         for name, attr in inspect.getmembers(instance):
-            if name.startswith('_'):
+            if name.startswith('_') or isinstance(attr, URI):
                 continue
+            elif hasattr(attr, "accessor"):
+                if attr.accessor:
+                    continue
             elif name in parent_members:
                 if attr is parent_members[name]:
                     continue
                 elif hasattr(attr, "__code__") and hasattr(parent_members[name], "__code__"):
                     if attr.__code__ is parent_members[name].__code__:
                         continue
+            elif inspect.ismethod(attr) and attr.__self__ is cls:
+                # it's a @classmethod
+                continue
 
             if isinstance(attr, MethodStub):
                 form[name] = to_json(attr.method(header, name))
@@ -62,20 +78,37 @@ class Meta(type):
     def __json__(cls):
         mro = cls.mro()
         if len(mro) < 2:
-            raise ValueError("Tinychain class must extend a subclass of State")
+            raise ValueError("TinyChain class must extend a subclass of State")
 
         parent = mro[1]
         return {str(uri(parent)): to_json(form_of(cls))}
 
 
+class Attribute(object):
+    def __init__(self, stub):
+        self.stub = stub
+
+    def __call__(self, *args, **kwargs):
+        raise RuntimeError(f"cannot call an Attribute; use tc.use(<class>) to generate attribute headers")
+
+    def method(self, header, name):
+        rtype = inspect.signature(self.stub).return_annotation
+        if rtype == inspect.Parameter.empty:
+            raise AttributeError(f"attribute {name} is missing a return annotation")
+
+        from tinychain.ref import MethodSubject
+        accessor = rtype(MethodSubject(header, name))
+        accessor.accessor = True
+        return accessor
+
+
 class MethodStub(object):
     def __init__(self, dtype, form):
         self.dtype = dtype
         self.form = form
 
     def __call__(self, *args, **kwargs):
-        raise RuntimeError(
-            "cannot call a MethodStub; use tc.use(<class>) for callable method references")
+        raise RuntimeError(f"cannot call a MethodStub; use tc.use(<class>) for callable method references")
 
     def method(self, header, name):
         return self.dtype(header, self.form, name)
