@@ -15,6 +15,9 @@ class Method(object):
     __uri__ = uri(op.Op)
 
     def __init__(self, header, form, name):
+        if not inspect.isfunction(form):
+            raise ValueError(f"reflection requires a Python method, not {form}")
+
         self.header = header
         self.form = form
         self.name = name
@@ -28,16 +31,22 @@ class Method(object):
     def dtype(self):
         return self.__class__.__name__
 
+    def subject(self):
+        if isinstance(self.header, State):
+            return ref.MethodSubject(self.header, self.name)
+        else:
+            return uri(self.header).append(self.name)
+
 
 class Get(Method):
     __uri__ = uri(op.Get)
 
     def __init__(self, header, form, name):
-        self.rtype = _get_rtype(form, State)
         Method.__init__(self, header, form, name)
 
     def __call__(self, key=None):
-        return self.rtype(ref.Get(uri(self.header).append(self.name), key))
+        rtype = _get_rtype(self.form, State)
+        return rtype(ref.Get(self.subject(), key))
 
     def __form__(self):
         cxt, args = _first_params(self)
@@ -70,7 +79,7 @@ class Put(Method):
         Method.__init__(self, header, form, name)
 
     def __call__(self, key, value):
-        return ref.Put(uri(self.header).append(self.name), key, value)
+        return ref.Put(self.subject(), key, value)
 
     def __form__(self):
         cxt, args = _first_params(self)
@@ -113,14 +122,14 @@ class Put(Method):
 class Post(Method):
     __uri__ = uri(op.Post)
 
-    def __init__(self, header, form, name):
-        self.rtype = _get_rtype(form, State)
-        Method.__init__(self, header, form, name)
+    def __call__(self, params=None, **kwargs):
+        if params is None:
+            params = kwargs
+        elif kwargs:
+            raise ValueError("Post method takes a single argument called `params`, or kwargs, not both")
 
-    def __call__(self, params):
-        rtype = inspect.signature(self.form).return_annotation
-        rtype = resolve_class(self.form, rtype, Nil)
-        return rtype(ref.Post(uri(self.header).append(self.name), params))
+        rtype = _get_rtype(self.form, State)
+        return rtype(ref.Post(self.subject(), params))
 
     def __form__(self):
         cxt, args = _first_params(self)
@@ -148,7 +157,7 @@ class Delete(Method):
         Method.__init__(self, header, form, name)
 
     def __call__(self, key=None):
-        return ref.Delete(uri(self.header).append(self.name), key)
+        return ref.Delete(self.subject(), key)
 
     def __form__(self):
         return Get.__form__(self)
