@@ -15,6 +15,9 @@ class Op(object):
     __uri__ = uri(op.Op)
 
     def __init__(self, form):
+        if not inspect.isfunction(form):
+            raise ValueError(f"reflection requires a Python callable, not {form}")
+
         self.form = form
 
     def __deps__(self):
@@ -25,6 +28,9 @@ class Op(object):
     def __json__(self):
         return {str(uri(self)): to_json(form_of(self))}
 
+    def __ref__(self):
+        raise RuntimeError("cannot reference a reflected Op; use Get, Put, Post, or Delete instead")
+
     def dtype(self):
         return self.__class__.__name__
 
@@ -187,8 +193,5 @@ def _maybe_first_arg(op):
                 args.append(cxt)
             else:
                 raise ValueError(f"{param} must be a {Context}, not {param.annotation}")
-        elif len(param_names) > 1:
-            raise ValueError(
-                f"optional first argument of {op.dtype()} must be 'cxt' or 'txn', not {param_names[0]}")
 
     return cxt, args
