@@ -1,4 +1,4 @@
-"""Tinychain `State` s, such as `Map`, `Tuple`, and `Op`."""
+"""TinyChain `State` s, such as `Map`, `Tuple`, and `Op`."""
 
 from tinychain import ref, reflect
 from tinychain.util import *
@@ -6,7 +6,7 @@ from tinychain.util import *
 
 class State(object):
     """
-    A Tinychain state, such as a `Chain` or `Op` or `Value`.
+    A TinyChain state, such as a `Chain` or `Op` or `Value`.
 
     Do not subclass `State` directly. Use a more specific type instead.
     """
@@ -26,7 +26,8 @@ class State(object):
 
     def __json__(self):
         form = form_of(self)
-        if isinstance(form, URI) or isinstance(form, ref.Op):
+
+        if reflect.is_ref(form):
             return to_json(form)
         else:
             return {str(uri(self)): [to_json(form)]}
@@ -34,8 +35,14 @@ class State(object):
     def __ns__(self, cxt):
         deanonymize(form_of(self), cxt)
 
+        if isinstance(self.__form__, URI):
+            self.__uri__ = self.__form__
+
     def __ref__(self, name):
-        return self.__class__(URI(name))
+        if hasattr(form_of(self), "__ref__"):
+            return self.__class__(get_ref(form_of(self), name))
+        else:
+            return self.__class__(URI(name))
 
     def __repr__(self):
         return f"{self.__class__.__name__}({form_of(self)})"
@@ -73,7 +80,22 @@ class State(object):
 
     def dtype(self):
         """Return the native base :class:`Class` of this `State`."""
-        return Class(ref.Get(uri(self) + "/class"))
+        return self._get("class", rtype=Class)
+
+    def is_none(self):
+        """Return `Bool(true)` if this `State` is :class:`Nil`."""
+
+        from .value import Bool
+        return self._get("is_none", rtype=Bool)
+
+    def is_some(self):
+        """
+        Return `Bool(true)` if this `State` is not :class:`Nil`.
+
+        This is defined as `self.is_none().logical_not()`.
+        """
+
+        return self.is_none().logical_not()
 
 
 class Map(State):
@@ -91,12 +113,25 @@ class Map(State):
         else:
             State.__init__(self, kwargs)
 
+    def __eq__(self, other):
+        return self.eq(other)
+
     def __getitem__(self, key):
         return self._get("", key)
 
     def __json__(self):
         return to_json(form_of(self))
 
+    def __ne__(self, other):
+        return self.ne(other)
+
+    def eq(self, other):
+        from .value import Bool
+        return self._post("eq", {"eq": other}, Bool)
+
+    def ne(self, other):
+        return self.eq(other).logical_not()
+
 
 class Tuple(State):
     """A tuple of `State` s."""
@@ -106,19 +141,38 @@ class Tuple(State):
     def __add__(self, other):
         return self.extend(other)
 
+    def __eq__(self, other):
+        return self.eq(other)
+
     def __json__(self):
         return to_json(form_of(self))
 
     def __getitem__(self, key):
         return self._get("", key)
 
+    def __ne__(self, other):
+        return self.ne(other)
+
+    def eq(self, other):
+        from .value import Bool
+        return self._post("eq", {"eq": other}, Bool)
+
+    def ne(self, other):
+        return self.eq(other).logical_not()
+
     def extend(self, other):
         return self._get("extend", other, Tuple)
 
+    def fold(self, initial_state, op):
+        return self._post("fold", {"value": initial_state, "op": op}, type(initial_state))
+
     def map(self, op):
         rtype = op.rtype if hasattr(op, "rtype") else State
         return self._post("map", {"op": op}, rtype)
 
+    def unpack(self, length):
+        yield from (self[i] for i in range(length))
+
     def zip(self, other):
         return self._get("zip", other, Tuple)
 
@@ -148,28 +202,29 @@ class Stream(State):
     def aggregate(self):
         return self._get("aggregate", rtype=Stream)
 
+    def first(self):
+        """Return the first item in this `Stream`, or `Nil` if the `Stream` is empty."""
+
+        return self._get("first", rtype=Scalar)
+
     def for_each(self, op):
         rtype = op.rtype if hasattr(op, "rtype") else State
         return self._post("for_each", Map(op=op), rtype)
 
-    def fold(self, initial_value, op):
-        return self._post("fold", Map(value=initial_value, op=op), type(initial_value))
+    def fold(self, initial_state, op):
+        return self._post("fold", Map(value=initial_state, op=op), type(initial_state))
 
     def map(self, op):
-        rtype = op.rtype if hasattr(op, "rtype") else State
-        return self._post("map", Map(op=op), rtype)
+        return self._post("map", Map(op=op), Stream)
 
 
 # User-defined object types
 
 class Class(State):
-    """A user-defined Tinychain class."""
+    """A user-defined TinyChain class."""
 
     __uri__ = uri(State) + "/object/class"
 
-    def __json__(self):
-        return {str(uri(Class)): to_json(form_of(self))}
-
 
 class Instance(State):
     """An instance of a user-defined :class:`Class`."""
