@@ -1,6 +1,8 @@
 """Utility data structures and functions."""
 
 import inspect
+import json
+import logging
 
 from collections import OrderedDict
 
@@ -43,15 +45,13 @@ class Context(object):
     def __getattr__(self, name):
         if name in self.form:
             value = self.form[name]
-            if isinstance(value, type):
-                from .state import Class
-                return Class(URI(name))
-            elif hasattr(value, "__ref__"):
+            if hasattr(value, "__ref__"):
                 return get_ref(value, name)
             else:
+                logging.info(f"context attribute {value} has no __ref__ method")
                 return URI(name)
         else:
-            raise ValueError(f"Context has no such value: {name}")
+            raise AttributeError(f"Context has no such value: {name}")
 
     def __json__(self):
         return to_json(list(self.form.items()))
@@ -67,6 +67,13 @@ class Context(object):
         else:
             self.form[name] = state
 
+    def __repr__(self):
+        data = list(self.form.keys())
+        return f"execution context with data {data}"
+
+    def is_defined(self, name):
+        return name in self.form
+
 
 def form_of(state):
     """Return the form of the given state."""
@@ -83,7 +90,14 @@ def form_of(state):
 def get_ref(subject, name):
     """Return a named reference to the given state."""
 
-    if hasattr(subject, "__ref__"):
+    if inspect.isclass(subject):
+        def ctr(*args, **kwargs):
+            return subject(*args, **kwargs)
+
+        ctr.__uri__ = URI(name)
+        ctr.__json__ = lambda: to_json(uri(ctr))
+        return ctr
+    elif hasattr(subject, "__ref__"):
         return subject.__ref__(name)
     else:
         return subject
@@ -91,7 +105,7 @@ def get_ref(subject, name):
 
 class URI(object):
     """
-    An absolute or relative link to a Tinychain state.
+    An absolute or relative link to a TinyChain state.
     
     Examples:
         .. highlight:: python
@@ -247,9 +261,16 @@ class URI(object):
         return str(self).startswith(str(prefix))
 
 
+def print_json(obj):
+    print(json.dumps(to_json(obj), indent=4))
+
+
 def requires(subject):
     """Return a set of the IDs of the dependencies required to resolve the given state."""
 
+    if inspect.isclass(subject):
+        return set()
+
     if hasattr(subject, "__deps__"):
         return subject.__deps__()
 
@@ -269,10 +290,11 @@ def requires(subject):
 def uri(subject):
     """Return the `URI` of the given state."""
 
+    if isinstance(subject, URI):
+        return subject
+
     if hasattr(subject, "__uri__"):
         return subject.__uri__
-    elif isinstance(subject, URI):
-        return subject
     elif hasattr(type(subject), "__uri__"):
         return uri(type(subject))
     else:
