@@ -1,6 +1,6 @@
 """:class:`Value` types such as :class:`Nil`, :class:`Number`, and :class:`String`."""
 
-from tinychain.ref import Ref
+from tinychain.ref import If, Ref
 from tinychain.state import Scalar
 from tinychain.util import form_of, to_json, uri, URI
 
@@ -70,6 +70,15 @@ class String(Value):
 
     __uri__ = uri(Value) + "/string"
 
+    def render(self, params=None, **kwargs):
+        if kwargs and params is not None:
+            raise ValueError("String.render accepts a Map or kwargs, not both:", params, kwargs)
+
+        if params:
+            return self._post("render", params, String)
+        else:
+            return self._post("render", kwargs, String)
+
 
 # Numeric types
 
@@ -78,12 +87,23 @@ class Number(Value):
 
     __uri__ = uri(Value) + "/number"
 
+    @classmethod
+    def max(cls, l, r):
+        return cls(If(l >= r, l, r))
+
+    @classmethod
+    def min(cls, l, r):
+        return cls(If(l <= r, l, r))
+
     def __abs__(self):
         return self.abs()
 
     def __add__(self, other):
         return self.add(other)
 
+    def __radd__(self, other):
+        return self.add(other)
+
     def __div__(self, other):
         return self.div(other)
 
@@ -105,15 +125,27 @@ class Number(Value):
     def __mul__(self, other):
         return self.mul(other)
 
-    def __ne__(self, other):
-        return self.ne(other)
-
-    def __radd__(self, other):
-        return self.add(other)
+    def __neg__(self):
+        return self * self.__class__(-1)
 
     def __rmul__(self, other):
         return self.mul(other)
 
+    def __rdiv__(self, other):
+        return self.__class__(other).div(self)
+
+    def __rpow__(self, other):
+        return self.__class__(other)**self
+
+    def __rtruediv__(self, other):
+        return self.__class__(other).div(self)
+
+    def __ne__(self, other):
+        return self.ne(other)
+
+    def __pow__(self, other):
+        return self.pow(other)
+
     def __sub__(self, other):
         return self.sub(other)
 
@@ -170,6 +202,11 @@ class Number(Value):
 
         return self._get("ne", other, Bool)
 
+    def pow(self, other):
+        """Raise `self` to the power of `other`."""
+
+        return self._get("pow", other, self.__class__)
+
     def sub(self, other):
         """Return the difference between `self` and `other`."""
 
@@ -181,6 +218,26 @@ class Bool(Number):
 
     __uri__ = uri(Number) + "/bool"
 
+    def logical_and(self, other):
+        """Boolean AND"""
+
+        return self._get("and", other, Bool)
+
+    def logical_not(self):
+        """Boolean NOT"""
+
+        return self._get("not", rtype=Bool)
+
+    def logical_or(self, other):
+        """Boolean OR"""
+
+        return self._get("or", other, Bool)
+
+    def logical_xor(self, other):
+        """Boolean XOR"""
+
+        return self._get("xor", other, Bool)
+
 
 class Complex(Number):
     """A complex number."""
@@ -233,51 +290,110 @@ class Int(Number):
 
     __uri__ = uri(Number) + "/int"
 
+    @classmethod
+    def max(cls):
+        return cls(2**(cls.size() - 1) - 1)
+
+    @classmethod
+    def min(cls):
+        return cls(-2**(cls.size() - 1) + 1)
+
+    @staticmethod
+    def size():
+        return 64
+
 
 class I16(Int):
     """A 16-bit integer."""
 
     __uri__ = uri(Int) + "/16"
 
+    @staticmethod
+    def size():
+        return 16
+
 
 class I32(Int):
     """A 32-bit integer."""
 
     __uri__ = uri(Int) + "/32"
 
+    @staticmethod
+    def size():
+        return 32
+
 
 class I64(Int):
     """A 64-bit integer."""
 
     __uri__ = uri(Int) + "/64"
 
+    @staticmethod
+    def size():
+        return 64
+
 
 class UInt(Number):
     """An unsigned integer."""
 
     __uri__ = uri(Number) + "/uint"
 
+    @classmethod
+    def max(cls):
+        return cls(2**cls.size() - 1)
+
+    @classmethod
+    def min(cls):
+        return cls(0)
+
+    @staticmethod
+    def size():
+        return 64
+
 
 class U8(UInt):
     """An 8-bit unsigned integer (a byte)."""
 
     __uri__ = uri(UInt) + "/8"
 
+    @staticmethod
+    def size():
+        return 8
+
 
 class U16(UInt):
     """A 16-bit unsigned integer."""
 
     __uri__ = uri(UInt) + "/16"
 
+    @staticmethod
+    def size():
+        return 16
+
 
 class U32(UInt):
     """A 32-bit unsigned integer."""
 
     __uri__ = uri(UInt) + "/32"
 
+    @staticmethod
+    def size():
+        return 32
+
 
 class U64(UInt):
     """A 64-bit unsigned integer."""
 
     __uri__ = uri(UInt) + "/64"
 
+    def __json__(self):
+        form = form_of(self)
+        if isinstance(form, int):
+            if form >= 2**I32.size():
+                return {str(uri(U64)): form}
+
+        return super().__json__()
+
+    @staticmethod
+    def size():
+        return 64
